/*
 * Copyright (C) 1998   Warren Jasper
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/*
 * a2dc.h
 */

/* General defines */
#define TRUE  1
#define FALSE 0
#define MAX_COUNT 1024

/* #define DEBUG  */

/* Default device major number */
#define DAS08_BASE     0x300     /* base addr (set on board) */
#define DAS08_IRQ          7     /*  irq      (set on board) */
#define DAC_GAIN0      BP_5_00V  /*  +/-  5V  (set on board) */
#define DAC_GAIN1      BP_5_00V  /*  +/-  5V  (set on board) */

#ifndef DEFAULT_MAJOR_DEV
#define DEFAULT_MAJOR_DEV  31    /* Default Major Device Number */
#endif

#define AD_CHANNELS         8    /*  8 Diff. Analog to Digial Chan */ 
#define DA_CHANNELS         2    /*  2 Digital to Analog Chan.     */
#define DIO_PORTS           3    /*  dio ports                     */
#define BOARD_SIZE       0x10    /* uses 16 bytes of registers     */

/* I/O Register locations */

/* NOTE: base_reg to be found dynamically at boot time. Not hard coded into driver */

#define LSB_AND_CHNLS    (base_reg)    /* A/D Data & Channel Register      */
#define MSB_DATA_BYTE    (base_reg+1)    
#define STATUS_REG       (base_reg+2)  /* Channel Mux Scan Limits Register */
#define GAIN_REG         (base_reg+3)  /* Programmable Gain Register       */
#define COUNTER_0_DATA   (base_reg+4)  /* Counter 0 Register               */
#define COUNTER_1_DATA   (base_reg+5)  /* Counter 1 Register               */
#define COUNTER_2_DATA   (base_reg+6)  /* Counter 2 Register               */
#define COUNTER_CONTROL  (base_reg+7)  /* Conter Control  Register         */
#define DA_CHAN0_LSB     (base_reg+8)  /* DAC 0 Low Byte                   */
#define DA_CHAN0_MSB     (base_reg+9)  /* DAC 0 High Byte                  */
#define DA_CHAN1_LSB     (base_reg+10) /* DAC 1 Low Byte                   */
#define DA_CHAN1_MSB     (base_reg+11) /* DAC 1 High Byte                  */
#define DIO_PORTA        (base_reg+12) /* Port A 8 bit I/O of 8255         */
#define DIO_PORTB        (base_reg+13) /* Port B 8 bit I/O of 8255         */
#define DIO_PORTC        (base_reg+14) /* Port C 4+4 bit of 8255           */
#define DIO_CNTRL_REG    (base_reg+15) /* Mode and Direction Control Reg.  */

/* Interrupt context data. Controls interrupt service handler */

typedef struct BOARD_REC {
  int  busy;           /* busy = TRUE, free = FALSE                */
  int  irq;            /* board irq                                */
  WORD base;           /* Base address of ADC board                */
  BYTE dio;            /* Upper 4 bits digital output              */
} BoardRec;

/* Values kept for each channel. */

typedef struct ADC_CHANNEL_REC {
  int  open;            /* Is channel device open()                 */
  LONG count;           /* Number of samples requested              */
  BYTE mode;            /* 0 =  Soft Trigger                        */
                        /* 2 = External Trigger                     */
  BYTE gain;            /* Voltage range                            */
  BYTE chan;            /* Mux number                               */
} ADC_ChanRec;

typedef struct DAC_CHANNEL_REC {
  int open;               /* Is channel device open()         */
  WORD value;             /* value of D/A Channel             */
  WORD addr;              /* address of the channel           */
  BYTE mode;              /* flags sent by open()             */
  BYTE gain;              /* Voltage range for output gain    */
} DAC_ChanRec;

typedef struct DIO_CHANNEL_REC {
  int  open;              /* Is channel device open()    */
  WORD value;             /* value of DIO Channel        */
  WORD addr;              /* address of the channel      */
  int mode;               /* mode for 8255               */
  BYTE f_flags;           /* flags sent by open()        */
} DIO_ChanRec;


/*************************************************************************
* STATUS_REG         base_reg+2             Status Register              *
**************************************************************************/

/* Read */
#define MUX0  0x01      /* Current multiplexor channel */
#define MUX1  0x02      /* Current multiplexor channel */
#define MUX2  0x04      /* Current multiplexor channel */
#define IRQ   0x08      /* 1 = positive edge detected  */
#define IP1   0x10      /* digial input line IP1       */
#define IP2   0x20      /* digial input line IP2       */
#define IP3   0x40      /* digial input line IP3       */
#define EOC   0x80      /* 1=A/D busy, 0=not busy      */

/* Write */
#define INTE  0x08     /* 1=enable interrupts, 0=disable */
#define OP1   0x10     /* digital output line OP1        */
#define OP2   0x20     /* digital output line OP2        */
#define OP3   0x40     /* digital output line OP3        */
#define OP4   0x80     /* digital output line OP4        */


/*************************************************************************
* Constants for dealing with the 8254 counters                           *
**************************************************************************/

#define MODE0 0x0
#define MODE1 0x2
#define MODE2 0x4
#define MODE3 0x6
#define MODE4 0x8
#define MODE5 0xa

#define C0 0x00
#define C1 0x40
#define C2 0x80

#define LATCH    0x00
#define LSBONLY  0x10
#define MSBONLY  0x20
#define LSBFIRST 0x30

#define S0       0x00
#define S1       0x02


