/*
 * Copyright (C) 1997  Warren Jasper
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 */


/***************************************************************************
 *
 *  test-das08.c
 *
 *  This program is used to test the CIO-DAS08-AOL Analog to Digital
 *  Linux loadable module.
 *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "das08.h"

/***************************************************************************
 *
 *  Global Data
 *
 ***************************************************************************/

char *DevName = "/dev/das08-AOL_adc00";
int  Gain     = UP_10_00V;
int  Mode     = ADC_SOFT_TRIGGER;
int  Count    = 1;
int  NoStop   = 0;
int  Print    = 1;
int  Status;

int fdDAC0, fdDAC1;          /* D/A file descriptors */
int fdADC;                   /* A/D file descriptors */
int fdDIOA, fdDIOB, fdDIOC;  /* DIO file descriptors */

void Usage( void )
{
    fprintf(stderr, "\n\
Usage: adcread \'options\'\n\
Options:\n\
              [-dev /dev/das08##]  - Specify device file.\n
              [-range (0-8)]     - Voltage input range (see manual)\n
              [-ct ##]           - Number of samples to read\n
              [-noprint]         - Don't print samples\n
              [-nostop]          - Sample forever\n
\n");
    exit(1);
}

void DoCommandLine(int argc, char **argv)
{
  int i = 1;

  while (i < argc) {
    if (strcmp(argv[i], "-dev") == 0) {
      i++;
      if (i == argc)
        Usage();
      else 
        DevName = argv[i];
    } else if (strcmp(argv[i], "-range") == 0) {
      i++;
      if (i == argc)
        Usage();
      else 
        Gain = atoi(argv[i]);
    } else if (strcmp(argv[i], "-ct") == 0) {
      i++;
      if (i == argc)
        Usage();
      else 
        Count = atoi(argv[i]);
    } else if (strcmp(argv[i], "-noprint") == 0) {
      Print = 0;
    } else if (strcmp(argv[i], "-nostop") == 0) {
      NoStop = 1;
    } else {
      Usage();
    }
    i++;
  }
}

void DoOpenDevices()
{

  char str[80];

  if (( fdADC = open(DevName, ADC_SOFT_TRIGGER)) < 0 ) {
    perror(str);
    printf("error opening device %s\n", DevName);
    exit(2);
  }

  strcpy(str, "/dev/das08-AOL_dac00");
  if (( fdDAC0 = open(str, O_RDWR)) < 0 ) {
    perror(str);
    printf("error opening device %s\n", str);
    exit(2);
  }

  strcpy(str, "/dev/das08-AOL_dac01");
  if (( fdDAC1 = open(str, O_RDWR)) < 0 ) {
    perror(str);
    printf("error opening device %s\n", str);
    exit(2);
  }

  strcpy(str, "/dev/das08-AOL_dio_A");
  if (( fdDIOA = open(str, O_RDWR)) < 0 ) {
    perror(str);
    printf("error opening device %s\n", str);
    exit(2);
  }

  strcpy(str, "/dev/das08-AOL_dio_B");
  if (( fdDIOB = open(str, O_RDWR)) < 0 ) {
    perror(str);
    printf("error opening device %s\n", str);
    exit(2);
  }

  strcpy(str, "/dev/das08-AOL_dio_C");
  if (( fdDIOC = open(str, O_RDWR)) < 0 ) {
    perror(str);
    printf("error opening device %s\n", str);
    exit(2);
  }
}

void testDIO()
{
  unsigned int value, bReg;
  char str[80];

  printf("***********************************************************\n");
  printf("* The program writes values on port A and reads them on   *\n");
  printf("* port B (8 bits), and writes on the low nibble of port C *\n");
  printf("* (4 bits) and reads them on the high nibble of port C.   *\n");
  printf("***********************************************************\n\n");

  ioctl(fdDIOA, DIO_SET_DIRECTION, PORT_OUTPUT);
  ioctl(fdDIOB, DIO_SET_DIRECTION, PORT_INPUT);
  ioctl(fdDIOC, DIO_SET_DIRECTION, LOW_PORT_OUTPUT);
  ioctl(fdDIOC, DIO_SET_DIRECTION, HIGH_PORT_INPUT);

  while (1) {
      printf("Enter value for port A in hex: ");
      scanf("%x", &value);
      write(fdDIOA, &value, 1);
      read(fdDIOB, &bReg, 1);
      printf("Port B value = %#x\n", bReg);
      printf("Enter value for port C in hex: ");
      scanf("%x", &value);
      value &= 0x0f;          /* mask off high nibble */
      write(fdDIOC, &value, 1);
      read(fdDIOC, &bReg, 1);
      bReg >>= 4;
      printf("Port C value = %#x\n", bReg);

      printf("\n Continue? ");
      scanf("%s", str);
      if (str[0] == 'n' || str[0] == 'N') return;
  }
}

void testDAC()
{
  int value;
  char str[80];

  while (1) {
    printf("Enter value for DAC0: ");
    scanf("%x", &value);
    write(fdDAC0, &value, 1);
    printf("Enter value for DAC1: ");
    scanf("%x", &value);
    write(fdDAC1, &value, 1);
    printf("\n Continue? ");
    scanf("%s", str);
    if (str[0] == 'n' || str[0] == 'N') return;
  }
}

void testADC()
{
  WORD value[10];
  char str[80];

  ioctl(fdADC, ADC_SET_GAINS, UP_5_00V);

  while (1) {
    read(fdADC, value, 1);
    printf("The value of %s is %#x\n", DevName, value[0]);
    printf("\n Continue? ");
    scanf("%s", str);
    if (str[0] == 'n' || str[0] == 'N') return;
  }
}

void Domenu()
{
  int choice;

  while(1) {
    system("/usr/bin/clear");
    printf("Select from the following choices:\n");
    printf("    1. Test Digital I/O.\n");
    printf("    2. Test Digital to Analog Converter.\n");
    printf("    3. Test Analog to Digital Converter.\n");
    printf("    4. Exit.\n");
    printf("\nOption: ");
   
    scanf("%d", &choice);
    switch(choice){
      case 1:  
        testDIO();
        break;
      case 2:  
        testDAC();
        break;
      case 3:  
        testADC();
        break;
      case 4:  
        return;
        break;
      default:
        break;
    }
  }    
}

int main(int argc, char **argv)
{
  DoCommandLine(argc, argv);
  DoOpenDevices();
  Domenu();
  close(fdDIOA);
  close(fdDIOB);
  close(fdDIOC);
  close(fdDAC0);
  close(fdDAC1);
  close(fdADC);
  return(1);
}


